/*
 * Copyright (c) 2007-2010 Sonatype, Inc.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 */
package org.sonatype.tycho.p2.publisher.plugin;

import java.io.File;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import org.apache.maven.RepositoryUtils;
import org.apache.maven.plugin.MojoExecutionException;
import org.apache.maven.project.DefaultDependencyResolutionRequest;
import org.apache.maven.project.DependencyResolutionException;
import org.apache.maven.project.DependencyResolutionResult;
import org.apache.maven.project.MavenProjectHelper;
import org.apache.maven.project.ProjectDependenciesResolver;
import org.codehaus.plexus.util.FileUtils;
import org.sonatype.aether.RepositorySystemSession;
import org.sonatype.aether.artifact.Artifact;
import org.sonatype.aether.graph.Dependency;
import org.sonatype.aether.graph.DependencyNode;
import org.sonatype.aether.util.graph.DefaultDependencyNode;
import org.sonatype.tycho.p2.eclipse.bridge.EclipseInstance;
import org.sonatype.tycho.p2.publisher.Publisher;
import org.sonatype.tycho.p2.publisher.PublisherFactory;
import org.sonatype.tycho.p2.publisher.PublisherResult;
import org.sonatype.tycho.p2.publisher.facade.ArtifactRepositoryIO;
import org.sonatype.tycho.p2.publisher.facade.MetadataRepositoryIO;

/**
 * @goal publish
 * @phase process-resources
 * @requiresDependencyResolution runtime
 */
public class PublishMojo
    extends WithinEclipseRunningProjectBasedMojo
{

    /**
     * @parameter expression="${project.groupId}:${project.artifactId}"
     * @required
     */
    protected String id;

    /**
     * @parameter expression="${project.build.directory}/tycho-p2/p2-update-site"
     * @required
     */
    protected File updateSiteDirectory;

    /**
     * @parameter expression="${tycho.p2.attachUpdateSiteXmls}" default-value="true"
     */
    private boolean attachUpdateSiteXmls;

    /**
     * @parameter expression="${repositorySystemSession}"
     * @readonly
     */
    private RepositorySystemSession repositorySystemSession;

    /**
     * @component
     */
    private ProjectDependenciesResolver dependencyResolver;

    /**
     * @component
     */
    private PublisherFactory publisherFactory;

    /**
     * @component
     */
    private MavenProjectHelper projectHelper;

    @Override
    protected void doWithEclipse( final EclipseInstance eclipse )
        throws MojoExecutionException
    {
        executePublisher( eclipse );
    }

    @Override
    protected Map<String, String> getLaunchProperties()
    {
        final Map<String, String> launchProperties = new HashMap<String, String>();
        launchProperties.putAll( super.getLaunchProperties() );
        // Better implementation that looks up the exported packages from the artifact
        launchProperties.put( "org.osgi.framework.system.packages.extra", "org.sonatype.tycho.p2.publisher.model;version=\"0.1.0\",org.sonatype.tycho.p2.publisher.model.artifact;version=\"0.1.0\",org.sonatype.tycho.p2.publisher.facade;version=\"0.1.0\"" );

        return launchProperties;
    }

    @Override
    protected Collection<ArtifactReference> getDefaultEclipsePlugins()
    {
        return Arrays.asList( new ArtifactReference[] { getPluginArtifact( "org.sonatype.tycho.p2", "p2-publisher-facade-imp" ) } );
    }

    protected PublisherResult executePublisher( final EclipseInstance eclipse )
        throws MojoExecutionException
    {
        try
        {
            final DefaultDependencyResolutionRequest resolution =
                new DefaultDependencyResolutionRequest( project, repositorySystemSession );
            final DependencyResolutionResult resolutionResult = dependencyResolver.resolve( resolution );

            final DependencyNode dependencyTree = fixRootNode( resolutionResult.getDependencyGraph() );

            final Publisher publisher = publisherFactory.create( eclipse );
            final PublisherResult publisherResult = publisher.publish( id, dependencyTree, project.getProperties() );

            FileUtils.deleteDirectory( updateSiteDirectory );

            final MetadataRepositoryIO metadataRepositoryIO = eclipse.getService( MetadataRepositoryIO.class );
            metadataRepositoryIO.write( updateSiteDirectory.toURI(), publisherResult.getInstallableUnits(), "tycho-p2-metadata", null );

            final ArtifactRepositoryIO artifactRepositoryIO = eclipse.getService( ArtifactRepositoryIO.class );
            artifactRepositoryIO.write( updateSiteDirectory.toURI(), publisherResult.getInstallableArtifacts(), "tycho-p2-artifacts", null );

            if ( attachUpdateSiteXmls )
            {
                attachUpdateSiteXmls();
            }

            return publisherResult;
        }
        catch ( final DependencyResolutionException exception )
        {
            throw new MojoExecutionException( "Cannot build project dependency tree", exception );
        }
        catch ( final Exception e )
        {
            throw new MojoExecutionException( "Cannot generate update site", e );
        }
    }

    // this changes the root node to be the project not a plain dumb node
    private DependencyNode fixRootNode( final DependencyNode root )
    {
        final DefaultDependencyNode newRoot = new DefaultDependencyNode( root );
        newRoot.getChildren().addAll( root.getChildren() );

        final Artifact artifact = RepositoryUtils.toArtifact( project.getArtifact() ).setFile( project.getFile() );
        newRoot.setDependency( new Dependency( artifact, "compile" ) );

        return newRoot;
    }

    private void attachUpdateSiteXmls()
    {
        projectHelper.attachArtifact( project, "xml", "p2Artifacts", new File( updateSiteDirectory, "artifacts.xml" ) );
        projectHelper.attachArtifact( project, "xml", "p2Content", new File( updateSiteDirectory, "content.xml" ) );
    }

}
